import warnings
from datetime import datetime

import typer
from rich.console import Console
from typing_extensions import Annotated

from cruxctl.command_groups.dataset_health.big_query_dataset_health_repo import (
    BiqQueryDatasetHealthRepo,
)
from cruxctl.command_groups.dataset_health.health_tree_data_adapter import (
    HealthTreeDataAdapter,
)
from cruxctl.command_groups.dataset_health.postgres_dataset_health_repo import (
    PostgresDatasetHealthRepo,
)
from cruxctl.command_groups.profile.profile import get_current_profile
from cruxctl.common.callbacks import datetime_to_date
from cruxctl.common.utils.data_adapter_utils import get_data_adapter
from cruxctl.common.models.data_format import DataFormat
from cruxctl.common.typer_constants import (
    PROFILE_OPTION,
    LISTING_LIMIT_OPTION,
    PROFILE_OPTION_TEST_ONLY,
    DATABASE_USER_OPTION,
    DATABASE_PASSWORD_OPTION,
    DATABASE_HOST_OPTION,
)

app = typer.Typer()

console = Console()

warnings.filterwarnings("ignore")


@app.command("get-all-processing-health")
def get_all_processing_health(
    delivery_deadline: Annotated[
        datetime,
        typer.Option(
            "--delivery-deadline",
            "-dd",
            formats=["%Y-%m-%d"],
            help="Filters on delivery deadlines that fall on this date",
            callback=datetime_to_date,
        ),
    ] = None,
    limit: Annotated[int, LISTING_LIMIT_OPTION] = 100,
    output_format: Annotated[
        DataFormat,
        typer.Option(
            "--output-format",
            "-o",
            case_sensitive=False,
            help="The output display format",
        ),
    ] = DataFormat.table,
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Lists all dataset processing health entries
    """
    data_adapter = get_data_adapter(output_format)
    if not profile:
        profile = get_current_profile()
    result = BiqQueryDatasetHealthRepo().get_all_processing_health(
        profile, data_adapter, delivery_deadline_date=delivery_deadline, limit=limit
    )

    console.print(result)


@app.command("get-processing-health")
def get_processing_health(
    dataset_id: Annotated[
        str, typer.Option("--dataset-id", "-d", help="Dataset ID to get for")
    ],
    delivery_deadline: Annotated[
        datetime,
        typer.Option(
            "--delivery-deadline",
            "-dd",
            formats=["%Y-%m-%d"],
            help="Filters on delivery deadlines that fall on this date",
            callback=datetime_to_date,
        ),
    ] = None,
    output_format: Annotated[
        DataFormat,
        typer.Option(
            "--output-format",
            "-o",
            case_sensitive=False,
            help="The output display format",
        ),
    ] = DataFormat.table,
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Gets the dataset processing health entries matching the dataset ID
    """
    data_adapter = get_data_adapter(output_format)
    if not profile:
        profile = get_current_profile()
    result = BiqQueryDatasetHealthRepo().get_processing_health_by_dataset_id(
        profile,
        data_adapter,
        dataset_id=dataset_id,
        delivery_deadline_date=delivery_deadline,
    )

    console.print(result)


@app.command("get-all-dispatch-health")
def get_all_dispatch_health(
    limit: Annotated[int, LISTING_LIMIT_OPTION] = 100,
    delivery_deadline: Annotated[
        datetime,
        typer.Option(
            "--delivery-deadline",
            "-dd",
            formats=["%Y-%m-%d"],
            help="Filters on delivery deadlines that fall on this date",
            callback=datetime_to_date,
        ),
    ] = None,
    output_format: Annotated[
        DataFormat,
        typer.Option(
            "--output-format",
            "-o",
            case_sensitive=False,
            help="The output display format",
        ),
    ] = DataFormat.table,
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Lists all dataset dispatch health entries
    """
    data_adapter = get_data_adapter(output_format)
    if not profile:
        profile = get_current_profile()
    result = BiqQueryDatasetHealthRepo().get_all_dispatch_health(
        profile, data_adapter, delivery_deadline_date=delivery_deadline, limit=limit
    )

    console.print(result)


@app.command("get-dispatch-health")
def get_dispatch_health(
    dataset_id: Annotated[
        str, typer.Option("--dataset-id", "-d", help="Dataset ID to get for")
    ],
    subscriber_id: Annotated[
        str, typer.Option("--subscriber-id", "-s", help="Subscriber ID to get for")
    ] = None,
    delivery_deadline: Annotated[
        datetime,
        typer.Option(
            "--delivery-deadline",
            "-dd",
            formats=["%Y-%m-%d"],
            help="Filters on delivery deadlines that fall on this date",
            callback=datetime_to_date,
        ),
    ] = None,
    output_format: Annotated[
        DataFormat,
        typer.Option(
            "--output-format",
            "-o",
            case_sensitive=False,
            help="The output display format",
        ),
    ] = DataFormat.table,
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Gets the dataset processing health entries matching either the dataset_id
    and/or the subscriber ID. At least one of
    the IDs must be provided.
    """
    if not dataset_id and not subscriber_id:
        raise typer.BadParameter("Either dataset_id or subscriber_id must be provided")

    data_adapter = get_data_adapter(output_format)
    if not profile:
        profile = get_current_profile()

    result = BiqQueryDatasetHealthRepo().get_dispatch_health(
        profile,
        data_adapter,
        dataset_id=dataset_id,
        subscriber_id=subscriber_id,
        delivery_deadline_date=delivery_deadline,
    )

    console.print(result)


@app.command("get-dataset-health-grouped-by-dataset")
def get_dataset_health_grouped_by_dataset(
    dataset_id: Annotated[
        str, typer.Option("--dataset-id", "-d", help="Dataset ID to get for")
    ] = None,
    subscriber_id: Annotated[
        str, typer.Option("--subscriber-id", "-s", help="Subscriber ID to get for")
    ] = None,
    delivery_deadline: Annotated[
        datetime,
        typer.Option(
            "--delivery-deadline",
            "-dd",
            formats=["%Y-%m-%d"],
            help="Filters on delivery deadlines that fall on this date",
            callback=datetime_to_date,
        ),
    ] = None,
    output_format: Annotated[
        DataFormat,
        typer.Option(
            "--output-format",
            "-o",
            case_sensitive=False,
            help="The output display format",
        ),
    ] = DataFormat.table,
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Gets the overall dataset health entries grouped by the dataset ID
    """
    data_adapter = (
        HealthTreeDataAdapter().bigquery_to_health_tree_by_dataset
        if output_format == DataFormat.tree
        else get_data_adapter(output_format)
    )
    if not profile:
        profile = get_current_profile()

    result = BiqQueryDatasetHealthRepo().get_dataset_health_grouped_by_dataset(
        profile,
        data_adapter,
        dataset_id=dataset_id,
        subscriber_id=subscriber_id,
        delivery_deadline_date=delivery_deadline,
    )

    console.print(result)


@app.command("get-dataset-health-grouped-by-subscriber")
def get_dataset_health_grouped_by_subscriber(
    dataset_id: Annotated[
        str, typer.Option("--dataset-id", "-d", help="Dataset ID to get for")
    ] = None,
    subscriber_id: Annotated[
        str, typer.Option("--subscriber-id", "-s", help="Subscriber ID to get for")
    ] = None,
    delivery_deadline: Annotated[
        datetime,
        typer.Option(
            "--delivery-deadline",
            "-dd",
            formats=["%Y-%m-%d"],
            help="Filters on delivery deadlines that fall on this date",
            callback=datetime_to_date,
        ),
    ] = None,
    output_format: Annotated[
        DataFormat,
        typer.Option(
            "--output-format",
            "-o",
            case_sensitive=False,
            help="The output display format",
        ),
    ] = DataFormat.table,
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Gets the overall dataset health entries grouped by the subscriber ID
    """
    data_adapter = (
        HealthTreeDataAdapter().bigquery_to_health_tree_by_subscriber
        if output_format == DataFormat.tree
        else get_data_adapter(output_format)
    )
    if not profile:
        profile = get_current_profile()
    result = BiqQueryDatasetHealthRepo().get_dataset_health_grouped_by_subscriber(
        profile,
        data_adapter,
        dataset_id=dataset_id,
        subscriber_id=subscriber_id,
        delivery_deadline_date=delivery_deadline,
    )

    console.print(result)


@app.command("get-all-notification-history")
def get_all_notification_history(
    limit: Annotated[
        int,
        typer.Option(
            "--limit",
            "-l",
            min=-1,
            help="Limit for the number of entries to return. Use -1 for unlimited",
        ),
    ] = 100,
    output_format: Annotated[
        DataFormat,
        typer.Option(
            "--output-format",
            "-o",
            case_sensitive=False,
            help="The output display format",
        ),
    ] = DataFormat.table,
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Lists all the notification history entries
    """
    data_adapter = get_data_adapter(output_format)
    if not profile:
        profile = get_current_profile()
    console.print(
        BiqQueryDatasetHealthRepo().get_all_notification_history(
            profile, data_adapter, limit=limit
        )
    )


@app.command("get-notification-history")
def get_notification_history(
    dataset_id: Annotated[str, typer.Argument(help="Dataset ID to get for")],
    output_format: Annotated[
        DataFormat,
        typer.Option(
            "--output-format",
            "-o",
            case_sensitive=False,
            help="The output display format",
        ),
    ] = DataFormat.table,
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Gets the notification history entries matching the dataset ID
    """
    data_adapter = get_data_adapter(output_format)
    if not profile:
        profile = get_current_profile()
    console.print(
        BiqQueryDatasetHealthRepo().get_notification_history_by_dataset_id(
            profile, data_adapter, dataset_id
        )
    )


@app.command("delete-all-processing-health")
def delete_all_processing_health(
    _: Annotated[str, PROFILE_OPTION_TEST_ONLY],
    user: Annotated[str, DATABASE_USER_OPTION],
    password: Annotated[str, DATABASE_PASSWORD_OPTION],
    host: Annotated[str, DATABASE_HOST_OPTION],
):
    """
    Deletes all processing health. Allowed only for test profile. Use with caution.
    """
    (
        PostgresDatasetHealthRepo(
            user=user, password=password, host=host, database="crux", port=5432
        ).delete_all_processing_health()
    )

    console.print("[green]All processing health entries deleted successfully[/green]")


@app.command("delete-all-dispatch-health")
def delete_all_dispatch_health(
    _: Annotated[str, PROFILE_OPTION_TEST_ONLY],
    user: Annotated[str, DATABASE_USER_OPTION],
    password: Annotated[str, DATABASE_PASSWORD_OPTION],
    host: Annotated[str, DATABASE_HOST_OPTION],
):
    """
    Deletes all dispatch health. Allowed only for test profile. Use with caution.
    """
    (
        PostgresDatasetHealthRepo(
            user=user, password=password, host=host, database="crux", port=5432
        ).delete_all_dispatch_health()
    )

    console.print("[green]All dispatch health entries deleted successfully[/green]")



import csv
from pathlib import Path
from typing import Callable, Optional

import typer
from google.cloud.bigquery.table import RowIterator

from cruxctl.command_groups.deadlines.dataset_deadline_client import (
    DatasetDeadlineClient,
)
from cruxctl.command_groups.deadlines.models.dataset_expected_delivery_deadline_columns import (
    DatasetExpectedDeliveryDeadlineJoinedColumn,
    DatasetExpectedDeliveryDeadlineColumnCamelCase,
)
from cruxctl.command_groups.deadlines.models.deadline_notification_snooze_units import (
    DeadlineNotificationSnoozeUnits,
)
from cruxctl.command_groups.deadlines.models.deadline_row import DeadlineRow
from cruxctl.common.models.data_format import DataFormat
from cruxctl.common.valid_timezones import VALID_TIMEZONES

EXCESSIVE_SNOOZE_THRESHOLD_DAYS = 14


def default_cron_validation(
    ctx: typer.Context,
    cron_element: str,
    lower_bound: int,
    upper_bound: int,
    cron_element_label: str,
    *args,
    asterisk_allowed: bool = True,
    extra_validation: Callable = None,
):
    if ctx.resilient_parsing:
        return

    if asterisk_allowed and cron_element == "*":
        return cron_element

    if extra_validation and extra_validation(cron_element, *args):
        return cron_element

    try:
        cron_element_int = int(cron_element)

        if cron_element_int < lower_bound or cron_element_int > upper_bound:
            raise typer.BadParameter(
                f"{cron_element_label} must be between {lower_bound}-{upper_bound}"
            )
    except ValueError:
        raise typer.BadParameter(f"{cron_element_label} is not a valid value")

    return cron_element


def deadline_minute_validation(ctx: typer.Context, deadline_minute: str):
    return default_cron_validation(
        ctx, deadline_minute, 0, 59, "Deadline minute", asterisk_allowed=False
    )


def deadline_hour_validation(ctx: typer.Context, deadline_hour: str):
    return default_cron_validation(ctx, deadline_hour, 0, 23, "Deadline hour")


def _validate_weekday_expression(deadline_day_of_the_month: str) -> bool:
    if deadline_day_of_the_month == "*W":
        return True

    if deadline_day_of_the_month.endswith("W"):
        try:
            day_of_month_value = int(deadline_day_of_the_month[:-1])

            if day_of_month_value < 1 or day_of_month_value > 31:
                raise typer.BadParameter("Deadline day of month must be between 1-31")

            return True
        except ValueError:
            raise typer.BadParameter("Deadline day of month is not a valid value")

    return False


def _validate_cron_interval(
    expression: str, lower_bound: int, upper_bound: int
) -> bool:
    if "-" in expression:
        try:
            split_expression = expression.split("-")

            if len(split_expression) != 2:
                raise typer.BadParameter(
                    f"Invalid cron interval expression: {expression}"
                )

            lower_input = int(split_expression[0])
            upper_input = int(split_expression[1])

            if (
                lower_input < lower_bound
                or upper_input > upper_bound
                or lower_input >= upper_input
            ):
                raise typer.BadParameter(
                    f"Expression {expression} must be a valid bound between "
                    f"{lower_bound}-{upper_bound}"
                )

            return True
        except ValueError:
            raise typer.BadParameter(f"Invalid cron interval expression: {expression}")

    return False


def deadline_day_of_the_month_validation(
    ctx: typer.Context, deadline_day_of_the_month: str
):
    return default_cron_validation(
        ctx,
        deadline_day_of_the_month,
        1,
        31,
        "Deadline day of month",
        extra_validation=_validate_weekday_expression,
    )


def deadline_month_validation(ctx: typer.Context, deadline_month: str):
    return default_cron_validation(ctx, deadline_month, 1, 12, "Deadline month")


def deadline_day_of_week_validation(ctx: typer.Context, deadline_day_of_week: str):
    return default_cron_validation(
        ctx,
        deadline_day_of_week,
        0,
        6,
        "Deadline day of week",
        0,
        6,
        extra_validation=_validate_cron_interval,
    )


def deadline_year_validation(ctx: typer.Context, deadline_year):
    if ctx.resilient_parsing:
        return

    if deadline_year != "*":
        raise typer.BadParameter("Currently the deadline year must be *")

    return deadline_year


def timezone_validation(ctx: typer.Context, timezone: Optional[str] = None):
    if ctx.resilient_parsing:
        return

    if timezone and timezone not in VALID_TIMEZONES:
        raise typer.BadParameter(f"Invalid timezone: {timezone}")

    return timezone


def _validate_csv_header(reader: csv.DictReader):
    header = set(reader.fieldnames)
    expected_header = {c.value for c in DatasetExpectedDeliveryDeadlineColumnCamelCase}

    if header != expected_header:
        raise typer.BadParameter(
            f"Invalid CSV header. CSV header must match the following:\n{expected_header}"
        )


def bigquery_to_deadline_row_set(bq_row_iterator: RowIterator) -> set[tuple]:
    results = set()

    for bq_row in bq_row_iterator:
        entry = (
            *(
                str(bq_row.get(column) or "")
                for column in bq_row.keys()
                if column
                != DatasetExpectedDeliveryDeadlineJoinedColumn.workflow_id.value
            ),
        )
        results.add(entry)

    return results


def _validate_row_formatting(ctx: typer.Context, row: DeadlineRow):
    deadline_minute_validation(ctx, row.deadline_minute)
    deadline_hour_validation(ctx, row.deadline_hour)
    deadline_day_of_the_month_validation(ctx, row.deadline_day_of_the_month)
    deadline_month_validation(ctx, row.deadline_month)
    deadline_day_of_week_validation(ctx, row.deadline_day_of_week)
    deadline_year_validation(ctx, row.deadline_year)
    timezone_validation(ctx, row.timezone)


def _validate_duplicate_row(row: DeadlineRow, duplicate_set: set, source_label: str):
    row_key = row.to_key()

    if row_key in duplicate_set:
        raise typer.BadParameter(
            f"Duplicate row found within the {source_label}: {row_key}"
        )

    duplicate_set.add(row_key)


def _validate_duplicate_dataset_ids(
    row: DeadlineRow, dataset_ids: set, source_label: str
):
    if row.dataset_id in dataset_ids:
        raise typer.BadParameter(
            f"Duplicate dataset ID found within the {source_label}: {row.dataset_id}"
        )


def csv_file_validation(
    ctx: typer.Context,
    profile: str,
    csv_file_path: Path,
    token: str = None,
):
    if ctx.resilient_parsing:
        return

    with open(csv_file_path) as csv_file:
        reader = csv.DictReader(csv_file)

        _validate_csv_header(reader)

        duplicate_database_list = DatasetDeadlineClient().get_all_deadlines(
            profile=profile, token=token, output_format=DataFormat.json
        )
        unique_dataset_ids = {item["datasetId"] for item in duplicate_database_list}
        duplicate_file_set = set()

        for row in reader:
            ddotm = (
                DatasetExpectedDeliveryDeadlineColumnCamelCase.deadline_day_of_the_month
            )
            try:
                deadline_row = DeadlineRow(
                    dataset_id=row[
                        DatasetExpectedDeliveryDeadlineColumnCamelCase.dataset_id.value
                    ],
                    deadline_minute=row[
                        DatasetExpectedDeliveryDeadlineColumnCamelCase.deadline_minute.value
                    ],
                    deadline_hour=row[
                        DatasetExpectedDeliveryDeadlineColumnCamelCase.deadline_hour.value
                    ],
                    deadline_day_of_the_month=row[(ddotm.value)],
                    deadline_month=row[
                        DatasetExpectedDeliveryDeadlineColumnCamelCase.deadline_month.value
                    ],
                    deadline_day_of_week=row[
                        DatasetExpectedDeliveryDeadlineColumnCamelCase.deadline_day_of_week.value
                    ],
                    deadline_year=row[
                        DatasetExpectedDeliveryDeadlineColumnCamelCase.deadline_year.value
                    ],
                    timezone=row[
                        DatasetExpectedDeliveryDeadlineColumnCamelCase.timezone.value
                    ],
                    file_frequency=row[
                        DatasetExpectedDeliveryDeadlineColumnCamelCase.file_frequency.value
                    ],
                    is_active=row[
                        DatasetExpectedDeliveryDeadlineColumnCamelCase.is_active.value
                    ],
                    is_excluded=row[
                        DatasetExpectedDeliveryDeadlineColumnCamelCase.is_excluded.value
                    ],
                )

                _validate_row_formatting(ctx, deadline_row)
                _validate_duplicate_row(deadline_row, duplicate_file_set, "CSV file")
                _validate_duplicate_dataset_ids(
                    deadline_row, unique_dataset_ids, "database"
                )
            except typer.BadParameter as e:
                message = f"CSV file validation failed at line {reader.line_num}.\n{e.message}"
                raise typer.BadParameter(message=message, ctx=ctx)


def is_snooze_duration_excessive(
    snooze_duration: int, snooze_units: DeadlineNotificationSnoozeUnits
):
    return (
        snooze_duration > EXCESSIVE_SNOOZE_THRESHOLD_DAYS
        if snooze_units == DeadlineNotificationSnoozeUnits.days
        else snooze_duration > EXCESSIVE_SNOOZE_THRESHOLD_DAYS * 24
    )



import json

from google.cloud.bigquery.table import RowIterator
from rich.table import Table, Column


def bigquery_to_richtable(bq_row_iterator: RowIterator) -> Table:
    table = Table(
        *[Column(field.name, overflow="fold") for field in bq_row_iterator.schema]
    )

    for bq_row in bq_row_iterator:
        table.add_row(*[str(bq_row.get(column)) for column in bq_row.keys()])

    return table


def bigquery_to_json(bq_row_iterator: RowIterator) -> str:
    results = []

    for bq_row in bq_row_iterator:
        entry = {column: str(bq_row.get(column)) for column in bq_row.keys()}
        results.append(entry)

    return json.dumps(results)


def json_to_rich_table(json_data: list) -> Table:
    table: Table
    if len(json_data) == 0:
        table = Table()
    else:
        table = Table(*[Column(key, overflow="fold") for key in json_data[0].keys()])
        for entry in json_data:
            table.add_row(*[str(entry[key]) for key in entry.keys()])
    return table



from typing import Optional

import warnings
from pathlib import Path

import typer
from rich.console import Console
from typing_extensions import Annotated

from cruxctl.command_groups.deadlines import deadlines_snooze
from cruxctl.command_groups.deadlines.dataset_deadline_client import (
    DatasetDeadlineClient,
)
from cruxctl.command_groups.profile.profile import get_current_profile
from cruxctl.common.models.data_format import DataFormat
from cruxctl.command_groups.deadlines.deadline_validations import (
    csv_file_validation,
)
from cruxctl.common.models.file_frequency import FileFrequency

from cruxctl.common.typer_constants import PROFILE_OPTION, LISTING_LIMIT_OPTION
from cruxctl.common.utils.api_utils import set_api_token

app = typer.Typer()
app.registered_commands += deadlines_snooze.app.registered_commands

console = Console()

warnings.filterwarnings("ignore")


@app.command("get-all")
def get_all(
    limit: Annotated[int, LISTING_LIMIT_OPTION] = 100,
    output_format: Annotated[
        DataFormat,
        typer.Option(
            "--output-format",
            "-o",
            case_sensitive=False,
            help="The output display format",
        ),
    ] = DataFormat.table,
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Lists all delivery deadline entries
    """
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)

    console.print(
        DatasetDeadlineClient().get_all_deadlines(
            profile=profile, token=token, limit=limit, output_format=output_format
        )
    )


@app.command()
def get(
    dataset_id: Annotated[str, typer.Argument(help="Dataset ID to get for")],
    output_format: Annotated[
        DataFormat,
        typer.Option(
            "--output-format",
            "-o",
            case_sensitive=False,
            help="The output display format",
        ),
    ] = DataFormat.table,
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Gets the delivery deadline entries matching the dataset ID
    """
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)

    console.print(
        DatasetDeadlineClient().get_all_deadlines(
            profile=profile,
            token=token,
            output_format=output_format,
            dataset_id=dataset_id,
        )
    )


@app.command()
def upsert(
    dataset_id: Annotated[
        str, typer.Argument(help="V1 Dataset ID of the deadline to insert")
    ],
    deadline_minute: Annotated[
        str,
        typer.Argument(
            help=(
                "Minute of the delivery deadline. Allowed values: 0-59. "
                "Leave 0 if unspecified."
            ),
        ),
    ],
    deadline_hour: Annotated[
        str,
        typer.Argument(
            help=("Hour of the delivery deadline. Allowed values: 0-23."),
        ),
    ],
    deadline_day_of_the_month: Annotated[
        str,
        typer.Argument(
            help=(
                "Day of the month delivery deadline. Allowed values: 1-31. "
                "Leave * if expected every day."
            ),
        ),
    ],
    deadline_month: Annotated[
        str,
        typer.Argument(
            help=(
                "Month of the delivery deadline. Allowed values: 1-12. "
                "Leave * if expected every month."
            ),
        ),
    ],
    deadline_day_of_week: Annotated[
        str,
        typer.Argument(
            help=(
                "Day of the week delivery deadline. Allowed values: 0-6. "
                "Leave * if expected every day."
            ),
        ),
    ],
    file_frequency: Annotated[
        FileFrequency,
        typer.Argument(
            help="Frequency of the file. Example values: daily, weekly, monthly, yearly"
        ),
    ],
    timezone: Annotated[
        Optional[str],
        typer.Argument(
            help=(
                "Timezone from which to calculate the delivery deadline. "
                "If unspecified it will use UTC. "
                "For the timezone format use the TZ identifier column found "
                "https://en.wikipedia.org/wiki/List_of_UTC_time_offsets."
            ),
        ),
    ] = "UTC",
    is_active: Annotated[
        bool,
        typer.Option(
            "--is-active",
            "-a",
            case_sensitive=False,
            help="Determines if the deadline is active",
        ),
    ] = True,
    is_excluded: Annotated[
        bool,
        typer.Option(
            "--is-excluded",
            "-e",
            case_sensitive=False,
            help="Determines if the deadline is excluded from Zendesk notifications",
        ),
    ] = False,
    delete_if_exists: Annotated[
        bool,
        typer.Option(
            "--delete-existing",
            "-d",
            case_sensitive=False,
            help="If set, deletes any existing deadlines for the dataset ID prior to upsert",
        ),
    ] = True,
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Upserts a single delivery deadline
    """
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)

    DatasetDeadlineClient().upsert_deadline(
        profile,
        token,
        dataset_id,
        deadline_minute,
        deadline_hour,
        deadline_day_of_the_month,
        deadline_month,
        deadline_day_of_week,
        file_frequency,
        timezone,
        is_active,
        is_excluded,
        delete_if_exists,
    )

    console.print(f"[green]Upserted deadline for dataset {dataset_id}[/green]")


@app.command("import")
def bulk_import(
    ctx: typer.Context,
    file_path: Annotated[
        Path,
        typer.Argument(
            help="""
    Path to CSV file to import.\n
    The CSV must have a header with the following columns:\n
    dataset_id, deadline_minute, deadline_hour, deadline_day_of_month,
    deadline_month, deadline_day_of_week, deadline_year, timezone.\n
    [bold yellow]WARNING[/bold yellow]: No validation is performed on the CSV file.
    It is assumed that the content is valid and entries are deduplicated.
    """
        ),
    ],
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Imports a CSV file with the delivery deadline entries.
    """
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)

    csv_file_validation(ctx, profile, file_path, token=token)

    DatasetDeadlineClient().bulk_insert_deadlines(
        profile, file_path=str(file_path), token=token
    )
    console.print("[green]Import completed successfully[/green]")


@app.command("export")
def bulk_export(
    file_path: Annotated[
        Path, typer.Argument(help="Path to CSV file to write data in locally")
    ],
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Exports a CSV file with all the delivery deadlines to the provided local path
    """
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)

    DatasetDeadlineClient().export_to_local_file(
        profile, file_path=str(file_path), token=token
    )
    console.print("[green]Export completed successfully[/green]")


@app.command()
def delete(
    id: Annotated[str, typer.Argument(help="ID to match for deletion")],
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Deletes the delivery deadline entries with the given ID
    """
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)

    DatasetDeadlineClient().delete_deadline_by_id(profile, token, id)
    console.print(f"[green]Deleted entry with ID: {id}[/green]")


@app.command("dataset-delete")
def dataset_delete(
    dataset_id: Annotated[str, typer.Argument(help="dataset_id to match for deletion")],
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Deletes the delivery deadline entries with the given dataset_id
    """
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)

    deadlines_by_dataset_id = DatasetDeadlineClient().get_all_deadlines(
        profile, token, dataset_id=dataset_id, output_format=DataFormat.json, limit=-1
    )

    console.print(deadlines_by_dataset_id)

    is_delete = typer.confirm(
        "Above are the entries matching with dataset_id, do you want to delete them?",
        default=False,
    )
    if is_delete:
        for deadline in deadlines_by_dataset_id:
            DatasetDeadlineClient().delete_deadline_by_id(
                profile, token, deadline["id"]
            )
        console.print(f"[green]Deleted entries with dataset ID: {dataset_id}[/green]")
    else:
        console.print("[red]No entries were deleted[/red]")



import requests

from cruxctl.common.utils.api_utils import (
    get_control_plane_url,
    get_api_headers,
    raise_for_status,
)

DEFAULT_API_TIMEOUT_SECONDS: int = 10


class LogsHandler:
    @staticmethod
    def get_logs(
        profile: str,
        api_token: str,
        dataset_id: str,
        type: str,
        page_token: str,
        page_size: int,
        **filter_criteria: dict,
    ) -> dict:
        """
        Calls the control plane API to retrieve logs for a dataset.

        :param profile: the Application Profile
        :param api_token: the API Bearer token
        :param dataset_id: ID of the dataset to get logs for
        :param type: pdk or dispatch
        :param page_token: Optional page token for next page of logs
        :param page_size: Optional page size for logs
        :param filter_criteria: A dictionary of optional filter parameters like
                                 export_id, execution_date, delivery_id, cdu_id.
        """
        base_url: str = get_control_plane_url(profile)
        url: str = f"{base_url}/datasets/{dataset_id}/logs/{type}"

        headers: dict = get_api_headers(api_token)

        # Building the request body dynamically based on the passed filter_criteria
        request_body = {
            "datasetId": dataset_id,
            "pageToken": page_token,
            "pageSize": page_size,
            "orderBy": "timestamp desc",
        }

        # Add the filter criteria if they are provided
        for key, value in filter_criteria.items():
            if value is not None:
                request_body[key] = value

        # Making the POST request
        response = requests.post(
            url=url,
            headers=headers,
            timeout=DEFAULT_API_TIMEOUT_SECONDS,
            json=request_body,
        )

        raise_for_status(response)
        return response.json()



from datetime import date
from typing import Callable

from google.cloud import bigquery

from cruxctl.command_groups.dataset_health.models.dataset_health_profile import (
    get_postgres_database,
    get_processing_health_table,
    get_dispatch_health_table,
    get_aggregated_by_dataset_status_view,
    get_aggregated_by_subscriber_status_view,
)
from cruxctl.common.utils.sql_utils import get_limit_expression


NOTIFICATION_HISTORY_TABLE = "delivery_recon.dataset_notification_history"


class BiqQueryDatasetHealthRepo:
    def __init__(self):
        self.client = bigquery.Client()

    def get_all_processing_health(
        self,
        profile: str,
        data_adapter: Callable,
        delivery_deadline_date: date,
        limit: int = 100,
    ):
        limit_expression = get_limit_expression(limit)
        where_expression = self.get_deadline_matches_date_expression(
            delivery_deadline_date
        )

        database = get_postgres_database(profile)
        table = get_processing_health_table(profile)

        query = (
            f'SELECT * FROM EXTERNAL_QUERY("{database}", '
            f'"SELECT dataset_id, '
            f"workflow_id, "
            f"delivery_deadline, "
            f"CAST(status AS TEXT), "
            f"delivery_id, "
            f"cdu_id, "
            f"scheduled_run, "
            f"{self._get_flattened_deadline_config_expression()}, "
            f"message, "
            f"CAST(created_at AS TEXT), "
            f"CAST(modified_at AS TEXT) "
            f"FROM {table} "
            f"{where_expression}"
            f"ORDER BY delivery_deadline DESC"
            f'{limit_expression};");'
        )

        query_job = self.client.query(query)
        rows = query_job.result()

        return data_adapter(rows)

    def _get_flattened_deadline_config_expression(self):
        return (
            "delivery_deadline_config->>'deadline_minute' AS deadline_minute, "
            "delivery_deadline_config->>'deadline_hour' AS deadline_hour, "
            "delivery_deadline_config->>'deadline_day_of_the_month' AS deadline_day_of_the_month, "
            "delivery_deadline_config->>'deadline_month' AS deadline_month, "
            "delivery_deadline_config->>'deadline_day_of_week' AS deadline_day_of_week, "
            "delivery_deadline_config->>'deadline_year' AS deadline_year, "
            "delivery_deadline_config->>'timezone' AS timezone"
        )

    def get_deadline_matches_date_expression(self, delivery_deadline_date: date) -> str:
        return (
            f"WHERE delivery_deadline::date = '{delivery_deadline_date}' "
            if delivery_deadline_date
            else ""
        )

    def get_processing_health_by_dataset_id(
        self,
        profile: str,
        data_adapter: Callable,
        dataset_id: str,
        delivery_deadline_date: date,
    ):
        database = get_postgres_database(profile)
        table = get_processing_health_table(profile)
        delivery_deadline_expression = (
            f" AND delivery_deadline::date = '{delivery_deadline_date}' "
            if delivery_deadline_date
            else ""
        )

        query = (
            f'SELECT * FROM EXTERNAL_QUERY("{database}", '
            f'"SELECT dataset_id, '
            f"workflow_id, "
            f"delivery_deadline, "
            f"CAST(status AS TEXT), "
            f"delivery_id, "
            f"cdu_id, "
            f"scheduled_run, "
            f"{self._get_flattened_deadline_config_expression()}, "
            f"message, "
            f"CAST(created_at AS TEXT), "
            f"CAST(modified_at AS TEXT) "
            f"FROM {table} "
            f"WHERE dataset_id = '{dataset_id}'{delivery_deadline_expression} "
            f'ORDER BY delivery_deadline DESC;");'
        )

        query_job = self.client.query(query)
        rows = query_job.result()

        return data_adapter(rows)

    def get_all_dispatch_health(
        self,
        profile: str,
        data_adapter: Callable,
        delivery_deadline_date: date,
        limit: int = 100,
    ):
        limit_expression = get_limit_expression(limit)
        where_expression = self.get_deadline_matches_date_expression(
            delivery_deadline_date
        )

        database = get_postgres_database(profile)
        table = get_dispatch_health_table(profile)

        query = (
            f'SELECT * FROM EXTERNAL_QUERY("{database}", '
            f'"SELECT dataset_id, '
            f"subscription_id, "
            f"delivery_deadline, "
            f"delivery_id, "
            f"subscriber_id, "
            f"cdu_id, "
            f"CAST(status AS TEXT), "
            f"{self._get_flattened_deadline_config_expression()}, "
            f"message, "
            f"CAST(created_at AS TEXT), "
            f"CAST(modified_at AS TEXT) "
            f"FROM {table} "
            f"{where_expression} "
            f"ORDER BY delivery_deadline DESC"
            f'{limit_expression};");'
        )

        query_job = self.client.query(query)
        rows = query_job.result()

        return data_adapter(rows)

    def get_dispatch_health(
        self,
        profile: str,
        data_adapter: Callable,
        dataset_id: str,
        subscriber_id: str,
        delivery_deadline_date: date,
    ):
        database = get_postgres_database(profile)
        table = get_dispatch_health_table(profile)

        where_expression = self._get_dataset_health_where_expression(
            dataset_id, subscriber_id, delivery_deadline_date
        )

        query = (
            f'SELECT * FROM EXTERNAL_QUERY("{database}", '
            f'"SELECT dataset_id, '
            f"subscription_id, "
            f"delivery_deadline, "
            f"delivery_id, "
            f"subscriber_id, "
            f"cdu_id, "
            f"CAST(status AS TEXT), "
            f"{self._get_flattened_deadline_config_expression()}, "
            f"message, "
            f"CAST(created_at AS TEXT), "
            f"CAST(modified_at AS TEXT) "
            f"FROM {table} "
            f"{where_expression} "
            f'ORDER BY delivery_deadline DESC;");'
        )

        query_job = self.client.query(query)
        rows = query_job.result()

        return data_adapter(rows)

    def _get_dataset_health_where_expression(
        self, dataset_id: str, subscriber_id: str, delivery_deadline_date: date
    ) -> str:
        if not dataset_id and not subscriber_id and not delivery_deadline_date:
            return ""

        conditions = []

        if dataset_id:
            conditions.append(f" dataset_id = '{dataset_id}'")

        if subscriber_id:
            conditions.append(f" subscriber_id = '{subscriber_id}' ")

        if delivery_deadline_date:
            conditions.append(f" delivery_deadline::date = '{delivery_deadline_date}'")

        return f"WHERE {' AND '.join(conditions)}"

    def get_dataset_health_grouped_by_dataset(
        self,
        profile: str,
        data_adapter: Callable,
        dataset_id: str,
        subscriber_id: str,
        delivery_deadline_date: date,
    ):
        database = get_postgres_database(profile)
        view = get_aggregated_by_dataset_status_view(profile)

        where_expression = self._get_aggregated_by_dataset_health_where_expression(
            dataset_id, subscriber_id, delivery_deadline_date
        )

        query = (
            f'SELECT * FROM EXTERNAL_QUERY("{database}", '
            f'"SELECT dataset_id, '
            f"delivery_deadline, "
            f"processing_status::text, "
            f"dispatch_statuses::text[], "
            f"aggregated_status::text, "
            f"subscription_ids::text[], "
            f"subscribers "
            f"FROM {view} "
            f'{where_expression};");'
        )

        query_job = self.client.query(query)
        rows = query_job.result()

        return data_adapter(rows)

    def _get_aggregated_by_dataset_health_where_expression(
        self, dataset_id: str, subscriber_id: str, delivery_deadline_date: date
    ) -> str:
        if not dataset_id and not subscriber_id and not delivery_deadline_date:
            return ""

        conditions = []

        if dataset_id:
            conditions.append(f" dataset_id = '{dataset_id}'")

        if subscriber_id:
            conditions.append(f" '{subscriber_id}' = ANY (subscribers) ")

        if delivery_deadline_date:
            conditions.append(f" delivery_deadline::date = '{delivery_deadline_date}'")

        return f"WHERE {' AND '.join(conditions)}"

    def get_dataset_health_grouped_by_subscriber(
        self,
        profile: str,
        data_adapter: Callable,
        dataset_id: str,
        subscriber_id: str,
        delivery_deadline_date: date,
    ):
        database = get_postgres_database(profile)
        view = get_aggregated_by_subscriber_status_view(profile)

        where_expression = self._get_dataset_health_where_expression(
            dataset_id, subscriber_id, delivery_deadline_date
        )

        query = (
            f'SELECT * FROM EXTERNAL_QUERY("{database}", '
            f'"SELECT subscriber_id, '
            f"dataset_id, "
            f"delivery_deadline, "
            f"processing_status::text, "
            f"dispatch_statuses::text[], "
            f"aggregated_status::text, "
            f"subscription_ids::text[]"
            f"FROM {view} "
            f'{where_expression};");'
        )

        query_job = self.client.query(query)
        rows = query_job.result()

        return data_adapter(rows)

    def get_all_notification_history(
        self, profile: str, data_adapter: Callable, limit: int = 100
    ):
        limit_expression = get_limit_expression(limit)

        database = get_postgres_database(profile)

        query = (
            f'SELECT * FROM EXTERNAL_QUERY("{database}", '
            '"SELECT CAST(id AS VARCHAR), dataset_id, CAST(delivery_deadline AS VARCHAR), '
            "cast(notification_type as VARCHAR), CAST(created_at AS VARCHAR), "
            "CAST(modified_at AS VARCHAR), "
            "notification_payload->>'file_frequency' as file_frequency, "
            "notification_payload->>'v2_dataset_id' as v2_dataset_id, "
            "notification_payload->>'delivery_mins_late' as delivery_mins_late, "
            "notification_payload->>'most_recent_delivery_time' as most_recent_delivery_time, "
            "notification_payload->>'workflow_id' as workflow_id, "
            "notification_payload->>'execution_time' as execution_time, "
            "notification_payload->>'cdu_id' as cdu_id, "
            "notification_payload->>'org_ids' as org_ids, "
            "notification_payload->'ticket_info'->>"
            "'ticket_creation_job_url' as ticket_creation_job_url, "
            "notification_payload->>'notification_hash' as notification_hash, "
            "notification_payload->>'deadline_minute' as deadline_minute, "
            "notification_payload->>'deadline_hour' as deadline_hour, "
            "notification_payload->>'deadline_day_of_the_month' as deadline_day_of_the_month, "
            "notification_payload->>'deadline_month' as deadline_month, "
            "notification_payload->>'deadline_day_of_week' as deadline_day_of_week, "
            "notification_payload->>'deadline_year' as deadline_year, "
            "notification_payload->>'timezone' as timezone "
            f'FROM {NOTIFICATION_HISTORY_TABLE}{limit_expression};");'
        )

        query_job = self.client.query(query)
        rows = query_job.result()

        return data_adapter(rows)

    def get_notification_history_by_dataset_id(
        self, profile: str, data_adapter: Callable, dataset_id: str
    ):
        database = get_postgres_database(profile)

        query = (
            f'SELECT * FROM EXTERNAL_QUERY("{database}", '
            '"SELECT CAST(id AS VARCHAR), dataset_id, CAST(delivery_deadline AS VARCHAR), '
            "cast(notification_type as VARCHAR),"
            "CAST(created_at AS VARCHAR), CAST(modified_at AS VARCHAR), "
            f"notification_payload->>'file_frequency' as file_frequency, "
            f"notification_payload->>'v2_dataset_id' as v2_dataset_id, "
            f"notification_payload->>'delivery_mins_late' as delivery_mins_late, "
            f"notification_payload->>'most_recent_delivery_time' as most_recent_delivery_time, "
            f"notification_payload->>'workflow_id' as workflow_id, "
            f"notification_payload->>'execution_time' as execution_time, "
            f"notification_payload->>'cdu_id' as cdu_id, "
            f"notification_payload->>'org_ids' as org_ids, "
            f"notification_payload->'ticket_info'->>'ticket_creation_job_url'"
            " as ticket_creation_job_url, "
            f"notification_payload->>'notification_hash' as notification_hash, "
            f"notification_payload->>'deadline_minute' as deadline_minute, "
            f"notification_payload->>'deadline_hour' as deadline_hour, "
            f"notification_payload->>'deadline_day_of_the_month' as deadline_day_of_the_month, "
            f"notification_payload->>'deadline_month' as deadline_month, "
            f"notification_payload->>'deadline_day_of_week' as deadline_day_of_week, "
            f"notification_payload->>'deadline_year' as deadline_year, "
            f"notification_payload->>'timezone' as timezone "
            f"FROM {NOTIFICATION_HISTORY_TABLE} "
            f"WHERE dataset_id = '{dataset_id}';\");"
        )

        query_job = self.client.query(query)
        rows = query_job.result()

        return data_adapter(rows)



from pathlib import Path

import platformdirs
import typer
from dotenv import set_key, load_dotenv, unset_key
from rich.console import Console

from cruxctl.command_groups.auth.auth_utilities import validate_profile, ProfileEnum
from cruxctl.common.utils.api_utils import read_config_file
import os

app = typer.Typer()
console = Console()
PROFILE_ENV_VAR = "CRUX_ACTIVE_PROFILE"


def load_active_profile_env_file():
    config_file_path = __get_active_profile_path()
    env_path = Path(config_file_path)
    if not env_path.exists():
        console.print(
            "[red]No active profile set. Defaulting to prod. "
            "Use `cruxctl profile set` command to default to a different(dev,stg) profile.[/red]"
        )
        env_path.touch()
    else:
        load_dotenv(config_file_path)


@app.command("set")
def set_current_profile(
    profile: str = typer.Argument(..., help="The profile to set - dev, stg or prod."),
):
    """
    Set the profile to use out of dev,stg or prod.
    """
    validate_profile(profile)
    config = read_config_file()
    if profile not in config:
        console.print(
            f"[red]Profile {profile} not found in config. "
            f"Please run 'cruxctl auth login -p' first.[/red]"
        )
        raise typer.Exit(code=1)
    set_active_profile_in_env(profile)
    load_active_profile_env_file()
    console.print(f"Profile set to: [green]{os.environ[PROFILE_ENV_VAR]}[/green]")


def __get_active_profile_path():
    crux_config_dir = platformdirs.user_config_dir("crux")
    return os.path.join(crux_config_dir, "active_profile.env")


def set_active_profile_in_env(profile: str):
    config_file_path = __get_active_profile_path()
    if not Path(config_file_path).exists():
        Path(config_file_path).touch()

    set_key(
        dotenv_path=config_file_path, key_to_set=PROFILE_ENV_VAR, value_to_set=profile
    )


def unset_active_profile_in_env():
    config_file_path = __get_active_profile_path()
    if not Path(config_file_path).exists():
        Path(config_file_path).touch()

    unset_key(dotenv_path=config_file_path, key_to_unset=PROFILE_ENV_VAR)


@app.command("get")
def get_current_profile():
    """
    Get the current profile.
    """
    load_active_profile_env_file()
    current_profile = os.getenv(PROFILE_ENV_VAR)
    if not current_profile:
        console.print(
            "[red]No profile set. Defaulting to prod. "
            "Use `cruxctl profile set` command to default to a different (dev,stg) profile.[/red]"
        )
        current_profile = ProfileEnum.PROD.value

    config = read_config_file()
    if current_profile not in config:
        console.print(
            f"[red]Credentials for Profile {current_profile} not found in config. "
            f"Please run 'cruxctl auth login --profile {current_profile}' first.[/red]"
        )
        raise typer.Exit(code=1)

    set_active_profile_in_env(current_profile)
    console.print(f"Current profile is: [green]{current_profile}[/green]")
    return current_profile



from typing import Callable

from cruxctl.common.data_adapters import bigquery_to_richtable, bigquery_to_json
from cruxctl.common.models.data_format import DataFormat


def get_data_adapter(data_format: DataFormat) -> Callable:
    if data_format == DataFormat.table:
        return bigquery_to_richtable

    if data_format == DataFormat.json:
        return bigquery_to_json

    raise ValueError(f"Unsupported data format: {data_format}")



import typer

from cruxctl.common.callbacks import validate_test_profile_only


CONTROL_PLANE_URL_OPTION = typer.Option(
    "--control-plane-url", "--cp", help="The URL of the control plane to use."
)

PROFILE_OPTION = typer.Option(
    "--profile",
    "-p",
    help="The application profile to use. Choose one of 3 profiles - dev, stg, prod."
    "This determines the backend resources accessed.",
)


PROFILE_OPTION_TEST_ONLY = typer.Option(
    "--profile",
    "-p",
    callback=validate_test_profile_only,
    help="The application profile to use. This determines the backend resources accessed.",
)


LISTING_LIMIT_OPTION = typer.Option(
    "--limit",
    "-l",
    min=-1,
    help="Limit for the number of entries to return. Use -1 for unlimited",
)

SORT_BY_OPTION = typer.Option("--sort-by", "-s", help="Sort the results by a field")
FILTERS_OPTION = typer.Option("--filters", "-f", help="Filter the results by a field")
PAGINATION_TOKEN_OPTION = typer.Option(
    "--pagination-token", "-pt", help="The token for pagination"
)
DATA_FORMAT_OPTION = typer.Option(
    "--output-format", "-o", case_sensitive=False, help="The output display format"
)

DATABASE_USER_OPTION = typer.Option("--user", "-u", help="Database username")
DATABASE_PASSWORD_OPTION = typer.Option("--password", "-pw", help="Database password")
DATABASE_HOST_OPTION = typer.Option("--host", "-h", help="Database host")



import requests

from cruxctl.command_groups.dataset.models.export_activity_type import (
    ExportActivityType,
)
from cruxctl.common.utils.api_utils import (
    get_control_plane_url,
    get_api_headers,
    raise_for_status,
)

DEFAULT_API_TIMEOUT_SECONDS: int = 10
DEFAULT_CLOUD_EVENT_SOURCE: str = "cruxctl"
DEFAULT_RERUN_QUEUE_KEY: str = "ongoing"


class DispatchRunHandler:
    @staticmethod
    def rerun_dispatch(
        profile: str,
        api_token: str,
        dataset_id: str,
        export_activity_id: str,
        queue_key: str = DEFAULT_RERUN_QUEUE_KEY,
        export_activity_type: ExportActivityType = ExportActivityType.SUCCESS,
    ) -> None:
        """
        Calls the control plane API to rerun a dispatch for a dataset.

        :param profile: the Application Profile
        :param api_token: the API Bearer token
        :param dataset_id: ID of the dataset to rerun
        :param export_activity_id: the export activity ID to rerun
        :param queue_key: the queue key to match
        :param export_activity_type: the export activity type to match
        """
        base_url: str = get_control_plane_url(profile)
        url: str = f"{base_url}/datasets/{dataset_id}/dispatch/rerun"

        headers: dict = get_api_headers(api_token)
        headers["source"] = DEFAULT_CLOUD_EVENT_SOURCE

        response = requests.request(
            method="POST",
            url=url,
            headers=headers,
            timeout=DEFAULT_API_TIMEOUT_SECONDS,
            json={
                "exportActivityId": export_activity_id,
                "queueKey": queue_key,
                "exportActivityType": export_activity_type.value,
            },
        )

        raise_for_status(response)



import typer

from cruxctl.command_groups.auth import auth
from cruxctl.command_groups.dataset import dataset
from cruxctl.command_groups.dataset_health import dataset_health
from cruxctl.command_groups.deadlines import deadlines
from cruxctl.command_groups.profile import profile

app = typer.Typer(rich_markup_mode="rich")
app.add_typer(
    deadlines.app,
    name="deadlines",
    help="Manage dataset delivery deadlines, "
    "Do not forget to set CRUX_API_TOKEN environment variable "
    "before running the commands.",
)
app.add_typer(
    dataset_health.app, name="dataset-health", help="Query dataset health info"
)
app.add_typer(
    dataset.app, name="dataset", help="Routines related to handling datasets."
)
app.add_typer(auth.app, name="auth", help="Authenticate with the Crux Control Plane.")
app.add_typer(
    profile.app, name="profile", help="Set the profile to use out of dev,stg or prod."
)

__version__ = "2.6.0"
# This file needs the version too.
auth.set_global_version(__version__)


@app.command(help="Gets the CLI version")
def version():
    print(f"CruxCtl Version: {__version__}")


if __name__ == "__main__":
    app()



import os
from typing import Optional

from cruxctl.common.utils.gcp_secrets_manager import get_secret_value


def set_openai_token(console):
    token = os.environ.get("OPENAI_API_KEY", None)
    if not token:
        secret_proj = "crux-data-science"
        secret_key = "OPENAI_API_KEY"
        console.rule("[bold red]Remove before public release", style="red")
        console.print(f"Fetching GCP secret: {secret_proj}/{secret_key}")
        console.rule(style="red")
        token = get_secret_value(secret_proj, secret_key)
        os.environ["OPENAI_API_KEY"] = token
    return token


def get_mixpanel_token() -> Optional[str]:
    """
    Get the Mixpanel token from the environment. If it can't find it, it returns None.
    :return: The Mixpanel token from the environment or empty string if it doesn't exist.
    """
    # The default IS NOT something that has to be protected by a password. It is simply
    # an identifier of which Mixpanel board to post to.
    return os.getenv("MIXPANEL_TOKEN", default="de3680bf905dfb9599cc22f1f4514557")



"""
Utility routines for the dataset command group.
"""

import asyncio
import os
import re
from pathlib import Path
from typing import List

import typer
from crux_odin.dict_utils import YAMLFileClosures, yaml_file_to_dict
from rich.console import Console

error_console = Console(stderr=True)


def get_yaml_file_closures(
    start_directory: str, yaml_files: List[str]
) -> List[List[str]]:
    """
    Returns lists of lists of parent/children in groups. These groups have to be coalesced
    and processed as a single YAML for each group.

    :param str start_directory: The directory to start looking for the YAML file and its parent.
    :param List[str] yaml_files: The YAML file to apply (with an optional parent). This must
    be a file reference and not a path (relative or absolute).
    """
    assert start_directory and isinstance(start_directory, str)
    assert yaml_files and isinstance(yaml_files, List)

    if not os.path.isdir(start_directory):
        error_console.print(f"Directory {start_directory} does not exist.")
        raise typer.Exit(1)
    shortened_yaml_list = []
    for file in yaml_files:
        if not re.compile(r"\.yaml$").search(str(file)):
            error_console.print(f"YAML file {file} must end in .yaml.")
            raise typer.Exit(1)
        if os.sep in str(file):
            # We are going to search below start_directory for the file so we don't need
            # the path portion. We don't allow two files with the same name so that shouldn't
            # be an issue.
            file = os.path.basename(file)
        shortened_yaml_list.append(file)

    yaml_full_paths = []
    for yaml_file in shortened_yaml_list:
        glob_generator = Path(start_directory).rglob(yaml_file)
        glob_path_list = [*glob_generator]
        assert (
            len(glob_path_list) <= 1
        ), f"Found more than one {yaml_file} in {start_directory}"
        if not glob_path_list:
            error_console.print(
                f"YAML file {yaml_file} not found in {start_directory}."
            )
            raise typer.Exit(1)
        yaml_full_paths.append(str(glob_path_list[0]))

    pathlib_list_of_list = asyncio.run(
        YAMLFileClosures(*yaml_full_paths, src_dir=start_directory).get_lists()
    )
    return pathlib_list_of_list


def verify_yaml_file_names_and_if_they_exist(yaml_files: List[str]):
    """
    Verifies that the file has the name '*.yaml' and that it exists.
    :param List[str] yaml_files: The YAML files to validate. These must be file references.
    """
    assert all(isinstance(s, str) for s in yaml_files)
    for file in yaml_files:
        if not re.compile(r"\.yaml$").search(file):
            error_console.print(f"YAML file {file} must end in .yaml.")
            raise typer.Exit(1)
        if not os.path.isfile(file):
            error_console.print(f"YAML file {file} does not exist.")
            raise typer.Exit(1)


def verify_parent_relationships_of_coalesced_files(yaml_files: List[str]):
    """
    Verifies that the YAML files passed have parent relationships between them. Prints
    warnings if they don't.
    :param List[str] yaml_files: YAML files to check for parent relationships.
    """
    assert (
        yaml_files
        and isinstance(yaml_files, list)
        and all(isinstance(s, str) for s in yaml_files)
        and all(re.compile(r"\.yaml$").search(file) for file in yaml_files)
    )

    file_path: dict = {}
    parent: dict = {}
    for yf in yaml_files:
        dict_from_yaml = yaml_file_to_dict(yf)
        if "id" not in dict_from_yaml:
            raise ValueError(f"YAML file {yf} does not have an 'id' field.")
        id = dict_from_yaml["id"]
        file_path[id] = yf
        file_stem = os.path.splitext(os.path.basename(yf))[0]
        if file_stem != id:
            error_console.print(
                f"YAML file {yf} has an 'id' field {id} that does not match the filename."
            )
        if "parent" in dict_from_yaml:
            no_path_parent = os.path.splitext(
                os.path.basename(dict_from_yaml["parent"])
            )[0]
            parent[id] = no_path_parent
    for k, v in parent.items():
        if v not in file_path:
            error_console.print(f"Parent {v} of {file_path[k]} does not exist.")
    if len(parent) != len(file_path) - 1:
        error_console.print(
            f"Not all YAML files of {yaml_files} have a parent relationship."
        )


def parse_cmd_line_yaml_lists(yamls: List[str]) -> List[List[str]]:
    """
    Parses the validate command line arguments, looks for parent/child relationships
    and returns lists of lists of files to concatenate together. The returned lists
    may contain relative or absolute paths.
    :param str yamls: The YAML files to validate. There are two modes: 1) Simple
    path names where parent/child relationships are given by separating the
    YAML files by commas and 2) First you give a directory. Then you give YAML
    file names below that directory. The command will search below the directory for
    the YAML file with that name and use it and also if the YAML file is a parent
    or a child, it will find the companion file and return that too.
    :return: List of lists of YAML files which can be absolute or relative paths,
    but the files have been verified to exist.
    """

    assert yamls and isinstance(yamls, list) and all(isinstance(s, str) for s in yamls)

    first_yaml = yamls[0]
    if os.path.isdir(first_yaml):
        start_directory = first_yaml
        yaml_files = yamls[1:]
        # get_yaml_files_closures() checks for file extensions and existence.
        return get_yaml_file_closures(start_directory, yaml_files)

    pathlib_list_of_list = []
    for yaml_file in yamls:
        if "," in yaml_file:
            split_yamls = yaml_file.split(",")
            verify_yaml_file_names_and_if_they_exist(split_yamls)
            # Now check the files being coalesced and warn if there aren't
            # parent relationships in the files.
            verify_parent_relationships_of_coalesced_files(split_yamls)
            pathlib_list_of_list.append(split_yamls)
        else:
            verify_yaml_file_names_and_if_they_exist([yaml_file])
            pathlib_list_of_list.append([yaml_file])
    return pathlib_list_of_list



from google.cloud import secretmanager


def get_secret_value(project_id, secret_id, version_id="latest"):
    # Create the Secret Manager client.
    client = secretmanager.SecretManagerServiceClient()

    # Build the resource name of the secret version.
    name = f"projects/{project_id}/secrets/{secret_id}/versions/{version_id}"

    # Access the secret version.
    response = client.access_secret_version(request={"name": name})

    # Return the decoded payload of the secret version.
    return response.payload.data.decode("UTF-8")



from datetime import datetime

import typer

from cruxctl.common.models.application_profile import ApplicationProfile


def datetime_to_date(ctx: typer.Context, datetime_value: datetime):
    if ctx.resilient_parsing:
        return

    return datetime_value.date() if datetime_value else None


def validate_test_profile_only(ctx: typer.Context, profile: ApplicationProfile):
    if ctx.resilient_parsing:
        return

    if profile != ApplicationProfile.local:
        raise typer.BadParameter(
            f"Only the test profile is allowed. Invalid profile: {profile}"
        )



from enum import Enum


class ApplicationProfile(str, Enum):
    """
    The command profile to determine which backend environment to use.
    """

    local = "local"
    dev = "dev"
    staging = "staging"
    prod = "prod"



import importlib
import json
import os.path
import platform
import re
import time
import warnings
from datetime import datetime
from enum import Enum
from pathlib import Path
from typing import Annotated, Any, List

import jsonschema
import pandas as pd
import requests
import typer
import yaml
from crux_odin.dataclass import (
    DeclaredField,
    DeclaredSchemaDef,
    Workflow,
    create_workflow,
)
from crux_odin.dict_utils import MergeDicts, yaml_file_to_dict
from crux_odin.validate_yaml import CustomValidations, validate_dict
from jinja2 import Environment, FileSystemLoader, select_autoescape
from mixpanel import Mixpanel
from pydantic import BaseModel
from rich.console import Console

import cruxctl
from cruxctl.command_groups.dataset.constants import CURATION_EVENT_TYPE, SchemaClass
from cruxctl.command_groups.dataset.dag_run_handler import DagRunHandler
from cruxctl.command_groups.dataset.dataset_utils import parse_cmd_line_yaml_lists
from cruxctl.command_groups.dataset.dispatch_run_handler import (
    DEFAULT_RERUN_QUEUE_KEY,
    DispatchRunHandler,
)
from cruxctl.command_groups.dataset.logs_handler import LogsHandler
from cruxctl.command_groups.dataset.models.export_activity_type import (
    ExportActivityType,
)
from cruxctl.command_groups.profile.profile import get_current_profile
from cruxctl.common.typer_constants import PROFILE_OPTION
from cruxctl.common.utils.api_utils import (
    get_control_plane_url,
    get_data_product_url,
    get_document_url,
    get_url_based_on_profile,
    get_user_info_by_token,
    set_api_token,
)
from cruxctl.common.utils.env_utils import get_mixpanel_token

app = typer.Typer()
console = Console()
error_console = Console(stderr=True)
warnings.filterwarnings("ignore")


def format_events_line(event: dict[str, Any], full: bool) -> str:
    """
    Format an event line for printing.
    :param event: The event to format.
    :param full: Whether to print the full event or not.
    """
    assert event and isinstance(event, dict)
    assert "time" in event and isinstance(event["time"], str)
    assert "type" in event and isinstance(event["type"], str)
    assert "source" in event and isinstance(event["source"], str)
    assert "subject" in event and isinstance(event["subject"], str)
    assert "data" in event and isinstance(event["data"], dict)
    assert "id" in event and isinstance(event["id"], str)
    assert "message" in event["data"] and isinstance(event["data"]["message"], str)

    # Extract dataset ID from subject
    dataset_id = event["subject"].split(":")[-1]
    data_copy = {
        k: v
        for k, v in event["data"].items()
        if k not in ["message", "eventType", "eventSource", "datasetId"]
    }
    data_copy["source"] = event["source"]

    result_str = '{:<30} {:<8} type={:<46} msg="{}"'.format(
        event["time"],
        dataset_id,
        event["type"],
        event["data"]["message"],
    )
    if full:
        result_str += f" metadata={data_copy}"
    return result_str


@app.command("events")
def events(
    dataset_id: Annotated[
        str, typer.Argument(help="The dataset ID currently being processed.")
    ],
    watch: Annotated[
        bool, typer.Option("--watch", "-w", help='Watch for changes (like "tail -f")')
    ] = False,
    profile: Annotated[
        str,
        typer.Option(help="profile to use: dev, stg or prod."),
    ] = None,
    full: Annotated[
        bool,
        typer.Option("--full", "-f", help="Give extended information about the event."),
    ] = False,
):
    """
    Prints out the log messages for the dataset ID being processed.
    :param str dataset_id: The dataset ID currently being processed.
    :param bool watch: Watch for changes (like "tail -f").
    """
    assert dataset_id and isinstance(dataset_id, str)
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)
    if not token:
        raise typer.Exit(1)

    previous_entry = None
    mixpanel_token = get_mixpanel_token()
    track_mixpanel_event(
        "cruxctl dataset events",
        {"Dataset ID": dataset_id},
        api_token=token,
        mixpanel_token=mixpanel_token,
        profile=profile,
    )
    while True:
        url = (
            f"{get_control_plane_url(profile)}"
            f"/events?filter=datasetId.eq.{dataset_id}"
        )
        if previous_entry:
            url += "&limit=300"
        response = requests.get(
            url,
            headers={"Accept": "application/json", "Authorization": f"Bearer {token}"},
        )
        if response.status_code != 200:
            error_console.print(
                f'Failed to get events for {dataset_id}."'
                f' Error: {json.loads(response.text)["message"]}'
            )
            raise typer.Exit(1)
        returned_entries = response.json()
        # Results are returned in REVERSE TIME ORDER.
        print_all_entries = True
        if previous_entry is not None:
            # Find the matching entry starting at the end.
            match_index = 0
            for entry in returned_entries:
                if entry == previous_entry:
                    break
                match_index += 1
            if match_index < len(returned_entries):
                print_all_entries = False
                for entry in reversed(returned_entries[0:match_index]):
                    print(format_events_line(entry, full))
        if print_all_entries:
            for entry in reversed(returned_entries):
                print(format_events_line(entry, full))
        if not watch:
            break
        if returned_entries:
            previous_entry = returned_entries[0]
        else:
            previous_entry = None
        # Polling sucks but we can't stream with REST calls. Convert to something that
        # can stream like gRPC or websockets later.
        time.sleep(3)


def _get_curation_schemas(
    token, workflow, dataset_id, data_product_id, profile
) -> dict[str, dict[str, DeclaredField]]:
    """
    Get the latest curation event for the dataset and return the declared schema
    """
    # get latest declared schema from the curation event and update the ODIN dataset spec
    curation_schemas = dict()

    pipeline_ids = [pipeline.id for pipeline in workflow.pipelines]
    latest_curation_events = _get_latest_curation_events(
        token, dataset_id, data_product_id, pipeline_ids, profile
    )

    for event in latest_curation_events:
        frame_id = event["data"]["message"]["frameName"]
        declared_curation = event["data"]["message"]["declaredSpecs"]

        vendor_table_name = declared_curation["vendor_table_name"]
        vendor_table_description = declared_curation["vendor_table_description"]
        vendor_schedule = declared_curation["vendor_schedule"]
        fields = declared_curation["fields"]
        declared_fields = [DeclaredField(**field) for field in fields]
        version = "1.0"

        output = {
            "vendor_table_name": vendor_table_name,
            "vendor_table_description": vendor_table_description,
            "vendor_schedule": vendor_schedule,
            "version": version,
            "fields": fields,
        }
        declared_schema_def = DeclaredSchemaDef(**output)

        pipeline = workflow.get_pipeline(frame_id)
        pipeline.set_declared_schema_def(declared_schema_def)
        curation_schemas[pipeline.id] = _format_schemas(declared_fields)

    return curation_schemas


def _format_schemas(declared_fields: List[DeclaredField]) -> dict[str, DeclaredField]:
    curation_output = dict()
    for field in declared_fields:
        curation_output[field.name] = field
    return curation_output


@app.command("update")
def update(
    input_file: Annotated[
        Path, typer.Option("--file", "-f", help="Path to the ODIN dataset spec")
    ],
    profile: Annotated[
        str,
        typer.Option(help="profile to use: local, dev, staging, prod. Default: prod"),
    ] = None,
    from_docs: Annotated[
        bool,
        typer.Option(
            "--from-docs", "-fd", help="Update the dataset schema with vendor docs"
        ),
    ] = False,
    accept_all: Annotated[
        bool,
        typer.Option(
            help="Override all types in the ODIN dataset spec with the vendor declared schema"
        ),
    ] = False,
):
    """
    Fetches vendor declared schema from a vendors supplier documentation.
    Fills out 'Catalog' of the ODIN dataset spec with the vendor declared schema.
    Through interactive CLI, allows user to configure final type.
    """
    if not from_docs:
        return
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)

    with open(input_file, "r") as file:
        yaml_workflow_dict = yaml.safe_load(file)
        workflow = create_workflow(yaml_workflow_dict, do_validation=False)

    dataset_id = yaml_workflow_dict["metadata"]["dataset_id"]
    data_product_id = yaml_workflow_dict["metadata"]["data_product_id"]
    curation_schemas = _get_curation_schemas(
        token,
        workflow,
        dataset_id,
        data_product_id,
        profile,
    )

    configured_schemas = _configure_curation_schemas(
        workflow, curation_schemas, accept_all
    )
    for pipeline_id, schema_def in configured_schemas.items():
        pipeline = workflow.get_pipeline(pipeline_id)
        pipeline.set_schema_def(schema_def)

    workflow_dict = workflow.model_dump(exclude_none=True, exclude_unset=True)
    mixpanel_token = get_mixpanel_token()
    track_mixpanel_event(
        "cruxctl dataset update",
        {
            "From Docs Used": from_docs,
            "Accept All Used": accept_all,
            "Dataset ID": dataset_id,
        },
        api_token=token,
        mixpanel_token=mixpanel_token,
        profile=profile,
    )
    with open(input_file, "w") as file:
        yaml.dump(workflow_dict, file)
    return


def _configure_curation_schemas(
    workflow: Workflow, curation_schemas: dict[str, Any], accept_all: bool
) -> dict[str, List]:
    # merge the declared schema with the existing schema
    configured_schemas = dict()
    schema_defs = workflow.get_all_pipeline_schema_defs()
    pipeline_ids = [pipeline.id for pipeline in workflow.pipelines]
    consolidated_dfs = _display_type_diffs(curation_schemas, schema_defs, pipeline_ids)

    for pipeline_id, df in consolidated_dfs.items():
        print(f"Resolving type differences for Pipeline: {pipeline_id}")
        resolved_df = _resolve_type_diffs(df, accept_all, pipeline_id)
        if not resolved_df:
            return
        configured_schemas[pipeline_id] = resolved_df
    return configured_schemas


def _check_uploaded_docs(
    token: str, dataset_id: str, data_product_id: str, profile: str
):
    url = f"{get_document_url(profile)}datasetId={dataset_id}"
    response = requests.get(
        url,
        headers={"Accept": "application/json", "Authorization": f"Bearer {token}"},
    )
    response.raise_for_status()
    returned_entries = response.json()
    if not returned_entries:
        print("No vendor documentation found for this dataset")
        redirect_url = (
            f"https://app.dev.cruxdata.com/data-products/"
            f"{data_product_id}/details/datasets/{dataset_id}/documentation"
        )
        print(f"Please upload the vendor documentation at: {redirect_url}")
        raise typer.Exit(code=1)


def _check_dataset_schema(
    token: str, dataset_id: str, data_product_id: str, profile: str
):
    url = f"{get_data_product_url(profile)}/{data_product_id}/datasets"
    response = requests.get(
        url,
        headers={"Accept": "application/json", "Authorization": f"Bearer {token}"},
    )
    response.raise_for_status()
    returned_entries = response.json()

    # check if dataset_id can be found from list of dataset_ids mapped to the data_product_id
    for entry in returned_entries:
        if entry["id"] == dataset_id:
            return

    print(
        "the dataset_id is not found for this product_id. Complete profiling schema at here"
    )
    redirect_url = f"https://app.dev.cruxdata.com/data-products/{data_product_id}/"
    print(f"Please complete profiling schema at: {redirect_url}")
    raise typer.Exit(code=1)


def _handle_no_curation_event(
    token: str, dataset_id: str, data_product_id: str, profile: str
):
    print(f"No curation event found for dataset: {dataset_id}")
    _check_dataset_schema(token, dataset_id, data_product_id, profile)
    _check_uploaded_docs(token, dataset_id, data_product_id, profile)
    raise typer.Exit(code=1)


def _get_latest_curation_events(
    token: str,
    dataset_id: str,
    data_product_id: str,
    pipeline_ids: List[str],
    profile: str,
) -> List[dict[str, Any]]:
    seen = set()
    latest_curation_event = []
    if not token:
        raise typer.Exit(1)

    url = (
        f"{get_control_plane_url(profile)}"
        f"/events?filter=subject.eq.crn:dataset:{dataset_id}"
    )
    print(url)
    response = requests.get(
        url,
        headers={"Accept": "application/json", "Authorization": f"Bearer {token}"},
    )
    response.raise_for_status()
    returned_entries = response.json()

    curation_events = []
    for event in returned_entries:
        if event["type"] == CURATION_EVENT_TYPE:
            curation_events.append(event)

    if not curation_events:
        _handle_no_curation_event(token, dataset_id, data_product_id, profile)
        raise typer.Exit(code=1)

    curation_events.sort(key=lambda x: x["time"], reverse=True)
    for event in curation_events:
        frame_name = event["data"]["message"]["frameName"]
        if frame_name in pipeline_ids and frame_name not in seen:
            seen.add(frame_name)
            latest_curation_event.append(event)
    print(latest_curation_event)
    return latest_curation_event


def _display_type_diffs(
    curation_schemas: dict[str, Any],
    yaml_schemas: dict[str, Any],
    pipeline_ids: List[str],
) -> dict[str, pd.DataFrame]:
    consolidated_dfs = dict()
    for idx, pipelind_id in enumerate(pipeline_ids):
        schema_def = yaml_schemas[idx]
        if pipelind_id in curation_schemas:
            curation_schema = curation_schemas[pipelind_id]
            consolidated_df = _consolidate_type_diffs_df(curation_schema, schema_def)
            consolidated_dfs[pipelind_id] = consolidated_df
    return consolidated_dfs


def _consolidate_type_diffs_df(
    curation_schema: dict[str, DeclaredField], yaml_schema: dict[str, Any]
) -> pd.DataFrame:
    yaml_columns = yaml_schema["fields"]
    yaml_column_names = [column["name"] for column in yaml_columns]
    consolidated = dict()

    for column in yaml_columns:
        column_name = column["name"]
        print(column_name)
        if column_name in curation_schema:
            curation_type = curation_schema[column_name].data_type
            consolidated[column_name] = [column["data_type"], curation_type]
        else:
            consolidated[column_name] = [column["data_type"], None]

    for curation_column in curation_schema.keys():
        if curation_column not in yaml_column_names:
            curation_type = curation_schema[curation_column].data_type
            consolidated[curation_column] = [None, curation_type]

    consolidated_df = pd.DataFrame.from_dict(
        consolidated,
        orient="index",
        columns=[SchemaClass.OBSERVED.value, SchemaClass.DECLARED.value],
    )
    return consolidated_df


def _resolve_type_diffs(
    consolidated_df: pd.DataFrame, accept_all: bool, pipeline_id: str
) -> List[dict[str, str]]:
    if consolidated_df is None or consolidated_df.empty:
        raise ValueError("Schema is empty. Exiting")
    console.print(consolidated_df)

    if accept_all:
        console.print(
            f"Overriding all types in the ODIN dataset spec for pipeline {pipeline_id}"
        )
        column_names = list(consolidated_df.index)
        resolved_types = list(consolidated_df[SchemaClass.DECLARED.value].values)

    else:
        column_names = list(consolidated_df.index)
        resolved_types = []
        for index, row in consolidated_df.iterrows():
            if row[SchemaClass.OBSERVED.value] == row[SchemaClass.DECLARED.value]:
                resolved_types.append(row[SchemaClass.OBSERVED.value])
                continue

            console.print(
                f"""Column Name: {index}
                | observed type: {row[SchemaClass.OBSERVED.value]}
                | declared type: {row[SchemaClass.DECLARED.value]}"""
                # noqa: E501
            )
            choice = typer.prompt(
                """Enter 1 to accept curation (DECLARED) type,
                2 to keep the existing (OBSERVED) type,
                3 for new type"""
                # noqa: E501
            )
            if int(choice) == 1:
                resolved_types.append(row[SchemaClass.DECLARED.value])
            elif int(choice) == 2:
                resolved_types.append(row[SchemaClass.OBSERVED.value])
            elif int(choice) == 3:
                new_type = typer.prompt("Enter the new type")
                resolved_types.append(new_type)
            else:
                new_type = typer.prompt("Enter the new type")
                resolved_types.append(new_type)

    configured_schema = []
    for column_name, resolved_type in zip(column_names, resolved_types):
        configured_schema.append(
            {"column_name": column_name, "data_type": resolved_type}
        )

    configured_df = pd.DataFrame(
        {SchemaClass.CONFIGURED.value: resolved_types}, index=column_names
    )
    resolved_df = pd.concat([consolidated_df, configured_df], axis=1)
    console.print(
        f"\nThis is configured schema based on your choices for pipeline: {pipeline_id}"
    )
    console.print(resolved_df)
    accept = typer.confirm("Do you accept this configured schema?")

    if accept:
        # Resolve the types
        console.print(
            f"Resolved types for pipeline: {pipeline_id}. Check the yaml for the update\n"
        )
        return configured_schema

    console.print(
        f"Not accepting the configured schema for pipeline: {pipeline_id}. Exiting\n"
    )
    raise typer.Abort()


def str_presenter(dumper, data):
    """
    Routine that puts quotes around strings that consist only of numbers but leaves
    everything else alone.
    """
    if re.compile("^[0-9]+$").match(data):
        if data == "08" or data == "09":
            data = re.sub("^0*", "", data)
        # If all digits, put quotes around it.
        return dumper.represent_scalar("tag:yaml.org,2002:str", data, style="'")
    return dumper.represent_scalar("tag:yaml.org,2002:str", data)


class ProfileValue(str, Enum):
    """
    Profile Value
    """

    local = "local"
    dev = "dev"
    staging = "staging"
    prod = "prod"


class ClusterValue(str, Enum):
    """
    Cluster Value
    """

    composer = "composer"
    oss = "oss"


class RegionValue(str, Enum):
    """
    Region Value
    """

    us_central1 = "us-central1"
    us_east1 = "us-east1"


class ShardValue(str, Enum):
    """
    Shard Value
    """

    zero = "0"
    one = "1"
    two = "2"
    three = "3"
    four = "4"
    five = "5"
    six = "6"
    a = "a"
    b = "b"
    c = "c"
    d = "d"
    e = "e"


class ApplyOdinParams(BaseModel):
    """
    Apply Odin Body
    """

    env: ProfileValue
    cluster: ClusterValue
    region: RegionValue
    shard: ShardValue

    class Config:
        use_enum_values = True


@app.command("apply")
def apply(
    file_list: Annotated[
        List[str],
        typer.Argument(
            help='File to apply or set of files to apply via a \
parent relationship if the files are separated by a comma. You can also give a directory \
as the first argument and a list of files below that directory to validate. It will look \
for matching parent or child files in that case. Example 1: "apply a.yaml b.yaml,c.yaml" \
- Apply a.yaml and combined b.yaml/c.yaml. Example 2: "apply ~/myyamls a.yaml c.yaml" - \
Apply a.yaml and c.yaml which exist below the directdory ~/myyamls. The command will \
search for the files a.yaml and c.yaml anywhere in the directory hierarchy below \
~/myyamls. In addition, suppose c.yaml has a "parent:" line pointing to b.yaml. Then \
b.yaml and c.yaml will be combined before they are applied.'
        ),
    ],
    quiet: Annotated[
        bool,
        typer.Option(
            "--quiet", "-q", help="Don't print out YAML files as they are applied."
        ),
    ] = False,
    profile: Annotated[
        ProfileValue,
        typer.Option(help="profile to use: local, dev, staging, prod. Default: prod"),
    ] = None,
    cluster: Annotated[
        ClusterValue,
        typer.Option(help="Airflow cluster to use: composer or oss. Default: oss"),
    ] = "oss",
    region: Annotated[
        RegionValue,
        typer.Option(
            help="Region to use: us-central1 or us-east1. Default: us-central1"
        ),
    ] = "us-central1",
    shard: Annotated[
        ShardValue,
        typer.Option(
            help="Shard to use: 0, 1, 2, 3, 4, 5, 6, a, b, c, d, e . Default: 0"
        ),
    ] = "a",
) -> None:
    """
    Apply an ODIN spec (possibly merged with its parent).

    :param Path start_directory: The directory to start looking for the YAML file and its parent.
    :param List[Path] yaml_files: The YAML file to apply (with an optional parent). This must
    be a file reference and not a path (relative or absolute).
    :param bool verbose: Print out YAML files as they are applied.
    """
    yaml.add_representer(str, str_presenter)
    yaml.representer.SafeRepresenter.add_representer(str, str_presenter)
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)
    if not token:
        raise typer.Exit(1)

    pathlib_list_of_list = parse_cmd_line_yaml_lists(file_list)
    for yaml_group in pathlib_list_of_list:
        dicts_to_coalesce = [yaml_file_to_dict(p) for p in yaml_group]
        dict_with_merged_yamls = MergeDicts(*dicts_to_coalesce)
        merged_dict_as_yaml = yaml.dump(dict(dict_with_merged_yamls))
        filename = os.path.basename(os.path.normpath(yaml_group[-1]))
        files = {
            "executionType": (None, "apply"),
            "file": (filename, merged_dict_as_yaml),
        }
        headers = {"Authorization": f"Bearer {token}"}
        url = f"{get_control_plane_url(profile)}/apply"
        shard = "0" if not cluster or cluster == "composer" else shard
        structured_body = ApplyOdinParams(
            env=profile, cluster=cluster, region=region, shard=shard
        )
        # We are passing the ApplyOdinParams as query parameters but this isn't ideal.
        # We should pass it in the body. However, I'm not quite sure how to do this
        # since we are already passing the file and we are possibly going to redesign
        # this interface or go to a pubsub interface so we'll deal with that later.
        response = requests.post(
            url, files=files, params=structured_body.model_dump(), headers=headers
        )
        dataset_name = dict_with_merged_yamls.get("id", "Unknown")
        mixpanel_token = get_mixpanel_token()
        if response.status_code not in [200, 201]:
            # I'm not sure why we get two different keys back when we get
            # an error. Maybe Python versus Java backends?
            track_mixpanel_event(
                "cruxctl dataset apply",
                {"Apply Result": False, "Dataset Name": dataset_name},
                api_token=token,
                mixpanel_token=mixpanel_token,
                profile=profile,
            )

            if "message" in response.text:
                message = json.loads(response.text)["message"]
            elif "detail" in response.text:
                message = json.loads(response.text)["detail"]
            else:
                message = response.text
            error_console.print(f"Failed to apply {yaml_group}. Error: {message}")
            raise typer.Exit(1)
        track_mixpanel_event(
            "cruxctl dataset apply",
            {"Apply Result": True, "Dataset Name": dataset_name},
            api_token=token,
            mixpanel_token=mixpanel_token,
            profile=profile,
        )
        if not quiet:
            console.print(f"Applied {yaml_group}")


@app.command("delete")
def delete(
    dataset_ids: Annotated[
        List[str],
        typer.Argument(help="Dataset ids to delete."),
    ],
    quiet: Annotated[
        bool,
        typer.Option(
            "--quiet", "-q", help="Don't print out dataset IDs aas they are printed."
        ),
    ] = False,
    profile: Annotated[
        str,
        typer.Option(help="profile to use: local, dev, staging, prod. Default: prod"),
    ] = None,
) -> None:
    """
    Deletes a set of dataset IDs.
    :param List[str] dataset_ids: The dataset IDs to delete.
    :param bool quiet: Don't print out dataset IDs as they are deleted.
    :param str profile: profile to use: local, dev, staging, prod. Default: prod.
    """
    assert (
        dataset_ids
        and isinstance(dataset_ids, list)
        and all(isinstance(s, str) for s in dataset_ids)
    )

    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)
    if not token:
        raise typer.Exit(1)

    for dataset_id in dataset_ids:
        url = f"{get_control_plane_url(profile)}/datasets/{dataset_id}"
        headers = {
            "Authorization": f"Bearer {token}",
        }
        response = requests.delete(url, headers=headers)
        if response.status_code != 200:
            error_console.print(
                f'Failed to delete {dataset_id}. Error: {json.loads(response.text)["message"]}'
            )
            raise typer.Exit(1)
        if not quiet:
            console.print(f"Deleted dataset ID {dataset_id}")


@app.command("validate")
def validate_yaml(
    file_list: Annotated[
        List[str],
        typer.Argument(
            help='File to validate or set of files to validate via \
parent relationship if the files are separated by a comma. You can also give a directory \
as the first argument and a list of files below that directory to validate. It will look \
for matching parent or child files in that case. Example 1: "validate a.yaml b.yaml,c.yaml" \
- Validate a.yaml and combined b.yaml/c.yaml. Example 2: "validate ~/myyamls a.yaml \
c.yaml" - Validate a.yaml and c.yaml which exist below the directory ~/myyamls. The \
command will search for the files a.yaml and c.yaml anywhere in the directory hierarchy \
below ~/myyamls. In addition, suppose c.yaml has a "parent:" line pointing to b.yaml. Then \
b.yaml and c.yaml will be combined before they are validated.'
        ),
    ],
    # This is opposite of what Unix commands usually do. When you do an "rm" or "cp"
    # Unix prints nothing. Unix usually only prints errors and is silent on success.
    # However, that isn't that useful here. People want feedback to know if the
    # validation succeeded.
    quiet: Annotated[
        bool,
        typer.Option(
            "--quiet", "-q", help="Don't print out YAML files as they are validated."
        ),
    ] = False,
    profile: Annotated[
        str,
        typer.Option(
            help="profile to use: local, dev, staging, prod. Default: prod."
            " This is only required when giving the --full option."
        ),
    ] = None,
    full: Annotated[
        bool,
        typer.Option(
            "--full",
            "-f",
            help="Run all validations including Crux specific custom validations."
            " This requires a call over the network to the control plane"
            " AND you have to give a profile environment.",
        ),
    ] = False,
) -> None:
    """
    Validate a YAML file (possibly merged with its parent). Since the parent and child
    YAML file can exist in separate directories, we give a start of the filesystem tree
    where to search for the YAML file and its parent.

    :param str start_directory: The directory to start looking for the YAML file and its parent.
    :param List[str] yaml_validation_files: The YAML files to validate (with an optional parent).
    These must be file references and not paths (relative or absolute).
    """
    assert (
        file_list
        and isinstance(file_list, list)
        and all(isinstance(s, str) for s in file_list)
    )

    pathlib_list_of_list = parse_cmd_line_yaml_lists(file_list)
    mixpanel_token = get_mixpanel_token()
    token = None
    if full:
        # token is necessary to call the POST request to do full validation.
        if not profile:
            profile = get_current_profile()
        token = set_api_token(console, profile)
        if not token:
            raise typer.Exit(1)
    for yaml_group in pathlib_list_of_list:
        dicts_to_coalesce = [yaml_file_to_dict(p) for p in yaml_group]
        dict_with_merged_yamls = MergeDicts(*dicts_to_coalesce)
        dataset_id = dict_with_merged_yamls.get("id", "Unknown")
        try:
            if full:
                headers = {
                    "Content-Type": "application/json",
                    "Authorization": f"Bearer {token}",
                }
                url = f"{get_control_plane_url(profile)}/validate_odin_spec"
                response = requests.post(
                    url, headers=headers, json=dict_with_merged_yamls
                )
                if response.status_code not in [200, 201]:
                    error_console.print(
                        f'Failed to apply {yaml_group}."'
                        f' Error: {json.loads(response.text)["message"]}'
                    )
                    raise typer.Exit(1)
            else:
                validate_dict(dict_with_merged_yamls)
                custom_validations = CustomValidations(dict_with_merged_yamls)
                custom_validations.validate_all()
            if token:
                track_mixpanel_event(
                    "cruxctl dataset validate",
                    {"Validation Result": True, "Dataset ID": dataset_id},
                    api_token=token,
                    mixpanel_token=mixpanel_token,
                    profile=profile,
                )
            if not quiet:
                console.print(f"Validation passed for {yaml_group}")
        except jsonschema.ValidationError as ex:
            error_console.print(f"Validation failed for {yaml_group}: {ex}")
            if token:
                track_mixpanel_event(
                    "cruxctl dataset validate",
                    {"Validation Result": False, "Dataset ID": dataset_id},
                    api_token=token,
                    mixpanel_token=mixpanel_token,
                    profile=profile,
                )
            raise typer.Exit(1)
        except Exception as ex:
            error_console.print(f"Validation failed for {yaml_group}: {ex}")
            if token:
                track_mixpanel_event(
                    "cruxctl dataset validate",
                    {"Validation Result": False, "Dataset ID": dataset_id},
                    api_token=token,
                    mixpanel_token=mixpanel_token,
                    profile=profile,
                )
            raise typer.Exit(1)


CRUX_CONFIG_FILE = "config.yaml"


# @app.command("init")
def init(
    yaml_output_file: Annotated[
        Path,
        typer.Argument(help="What file to dump the new YAML file in"),
    ],
    version: Annotated[
        str,
        typer.Option(
            help="Workflow version. Currently unimplemented."
            " You currently always get the latest version."
        ),
    ] = "1.2.0",
    dataset_name: Annotated[
        str,
        typer.Option(help="Dataset name. If not given taken from output file name."),
    ] = "",
    data_product_name: Annotated[
        str,
        typer.Option(
            help="Data Product name. If not given taken from output file name."
        ),
    ] = "",
    profile: Annotated[
        str,
        typer.Option(help="profile to use: local, dev, staging, prod. Default: prod"),
    ] = None,
) -> None:
    """
    Create a new YAML file with the given dataset name.
    Args:
        version: Version of YAML file to create (default is latest 1.2.0).
        dataset_name: Name of the new dataset. This will be created in Crux.
        Default: from filename.
        data_product_name: Name of the new data product. This will be created in Crux.
        Default: from filename.
        yaml_output_file: The name of the output file to dump the YAML to.
        profile: profile to use: dev, staging, prod. Default: prod.

    Returns:
        None
    """
    if not profile:
        profile = get_current_profile()
    token = set_api_token(console, profile)
    if not token:
        raise typer.Exit(1)

    # Get payload to get the email then get the org ID.
    user_info = get_user_info_by_token(token, profile=profile)
    fetched_org_id = user_info["orgId"]
    console.print(f'Using org ID "{fetched_org_id}"')

    # See if data product is there. If not create it.
    file_path_basename = os.path.splitext(os.path.basename(yaml_output_file))[0]
    if not data_product_name:
        data_product_name = file_path_basename
    url = f"{get_url_based_on_profile(profile)}/catalog/data-products"
    headers = {
        "Authorization": f"Bearer {token}",
        "Content-Type": "application/json",
        "Accept": "application/json",
    }
    params = {"filter": f"name.eq.{data_product_name}"}
    console.print(f'Checking if data product "{data_product_name}" exists')
    response = requests.get(url, headers=headers, params=params)
    response.raise_for_status()
    if not response.json():
        console.print("It doesn't. Creating it.")
        body = {
            "name": f"{data_product_name}",
            "orgId": f"{fetched_org_id}",
            "subscriptionType": "EDP",
        }
        url = f'{get_url_based_on_profile(profile, version="v4")}/data-products'
        response = requests.post(url, headers=headers, json=body)
        if not 200 <= response.status_code <= 201:
            error_console.print(
                f'Error creating data product "{data_product_name}": {response.text}'
            )
            raise typer.Exit(1)
        data_product_id = response.json()["id"]
        console.print(
            f'Created data product "{data_product_name}" with ID "{data_product_id}"'
        )
    else:
        data_product_id = response.json()[0]["id"]
        console.print(
            f'Data product "{data_product_name}" with ID "{data_product_id}"'
            " already exists. Using it."
        )

    # Create the dataset
    if not dataset_name:
        dataset_name = file_path_basename
    url = f"{get_url_based_on_profile(profile)}/datasets"
    headers = {
        "Authorization": f"Bearer {token}",
        "Content-Type": "application/json",
        "Accept": "application/json",
    }
    body = {"name": dataset_name, "ownerIdentityId": fetched_org_id}
    response = requests.post(url, headers=headers, json=body)
    response.raise_for_status()
    dataset_id = response.json()["id"]
    console.print(f'Created dataset "{dataset_name}" with ID "{dataset_id}"')

    # We need to map the dataset id and data product id.
    url = (
        get_url_based_on_profile(profile, version="v4", swap_ops_version=True)
        + "/data-products/maps"
    )
    body = {"datasetId": dataset_id, "dataProductId": data_product_id}
    response = requests.post(url, headers=headers, json=body)
    response.raise_for_status()
    console.print(
        f'Mapped dataset "{dataset_name}" "{dataset_id}" to data product ID "{data_product_id}"'
    )

    # Get the Jinja2 template
    template_paths = ["templates", "cruxctl/templates"]
    with importlib.resources.as_file(
        importlib.resources.files(cruxctl) / "templates"
    ) as p:
        template_paths.append(str(p))
    env = Environment(
        loader=FileSystemLoader(template_paths), autoescape=select_autoescape()
    )
    template = env.get_template("dataset_init.yaml.j2")
    rendered_yaml = template.render(
        dataset_id=dataset_id, data_product_id=data_product_id, org_id=fetched_org_id
    )
    with open(yaml_output_file, "w") as f:
        f.write(rendered_yaml)
    console.print(f"Created {yaml_output_file}")


@app.command("run-dag")
def run_dag(
    dataset_id: Annotated[
        str,
        typer.Argument(help="The dataset ID to run the DAG for."),
    ],
    dag_run_id: Annotated[
        str,
        typer.Option("--dag-run-id", "-d", help="A unique ID for the DAG run."),
    ],
    logical_date: Annotated[
        datetime,
        typer.Option(
            "--logical-date",
            "-l",
            formats=["%Y-%m-%dT%H:%M:%S"],
            help="The datetime to schedule the DAG run for.",
        ),
    ],
    note: Annotated[
        str, typer.Option("--note", "-n", help="Optional note for the DAG run.")
    ] = None,
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Invokes a new DAG run for the provided dataset. This kicks off the run in the background. The
    actual execution status can be retrieved by querying the dataset events.
    """
    if not profile:
        profile = get_current_profile()
    token: str = set_api_token(console, profile)
    if not token:
        raise typer.BadParameter("Failed to read Crux API token")

    try:
        mixpanel_token = get_mixpanel_token()
        track_mixpanel_event(
            "cruxctl dataset run-dag",
            {"Dataset ID": dataset_id, "Dag Run ID": dag_run_id},
            api_token=token,
            mixpanel_token=mixpanel_token,
            profile=profile,
        )
        DagRunHandler.run_dag(
            profile, token, dataset_id, dag_run_id, logical_date, note
        )
    except Exception as e:
        error_console.print(f"[red]Failed to invoke DAG run API: {e}[/red]")
        raise typer.Exit(code=1)

    console.print("[green]DAG run successfully invoked.[/green]")


@app.command("rerun-dag")
def rerun_dag(
    dataset_id: Annotated[
        str,
        typer.Argument(help="The dataset ID to run the DAG for."),
    ],
    dag_run_id: Annotated[
        str,
        typer.Option("--dag-run-id", "-d", help="A unique ID for the DAG run."),
    ],
    dry_run: Annotated[
        bool,
        typer.Option("--dry-run", help="True if this is a dry rerun."),
    ] = False,
    only_failed: Annotated[
        bool,
        typer.Option("--only-failed", help="True to only rerun failed tasks."),
    ] = False,
    only_running: Annotated[
        bool,
        typer.Option(
            "--only-running", help="True to only rerun already running tasks."
        ),
    ] = False,
    task_ids: Annotated[
        List[str],
        typer.Option(
            "--task-id",
            help="The task ID to rerun. Multiple option definitions "
            "can be passed in. If none provided, all tasks will be rerun.",
        ),
    ] = None,
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Invokes are DAG rerun for the provided dataset and run ID. This kicks off the rerun in the
    background. The actual execution status can be retrieved by querying the dataset events.
    """
    if not profile:
        profile = get_current_profile()
    token: str = set_api_token(console, profile)
    if not token:
        raise typer.BadParameter("Failed to read Crux API token")

    try:
        mixpanel_token = get_mixpanel_token()
        track_mixpanel_event(
            "cruxctl dataset rerun-dag",
            {"Dataset ID": dataset_id, "Dag Run ID": dag_run_id},
            api_token=token,
            mixpanel_token=mixpanel_token,
            profile=profile,
        )
        DagRunHandler.rerun_dag(
            profile,
            token,
            dataset_id,
            dag_run_id,
            dry_run,
            only_failed,
            only_running,
            task_ids,
        )
    except Exception as e:
        error_console.print(f"[red]Failed to invoke DAG rerun API: {e}[/red]")
        raise typer.Exit(code=1)

    console.print("[green]DAG rerun successfully invoked.[/green]")


@app.command("rerun-dispatch")
def rerun_dispatch(
    dataset_id: Annotated[
        str,
        typer.Argument(help="The dataset ID to rerun the dispatch for."),
    ],
    export_activity_id: Annotated[
        str,
        typer.Option(
            "--export-activity-id",
            "-e",
            help="The matching export activity ID to rerun.",
        ),
    ],
    queue_key: Annotated[
        str,
        typer.Option("--queue-key", help="The queue for the rerun."),
    ] = DEFAULT_RERUN_QUEUE_KEY,
    export_activity_type: Annotated[
        ExportActivityType,
        typer.Option(
            "--export-activity-type", help="The export activity type to match."
        ),
    ] = ExportActivityType.SUCCESS,
    profile: Annotated[str, PROFILE_OPTION] = None,
):
    """
    Invokes are dispatch rerun for the provided dataset and export ID. This kicks off the rerun in
    the background. The actual execution status can be retrieved by querying the dataset events.
    """
    if not profile:
        profile = get_current_profile()
    token: str = set_api_token(console, profile)
    if not token:
        raise typer.BadParameter("Failed to read Crux API token")

    try:
        mixpanel_token = get_mixpanel_token()
        track_mixpanel_event(
            "cruxctl dataset rerun-dispatch",
            {"Dataset ID": dataset_id},
            api_token=token,
            mixpanel_token=mixpanel_token,
            profile=profile,
        )
        DispatchRunHandler.rerun_dispatch(
            profile,
            token,
            dataset_id,
            export_activity_id,
            queue_key,
            export_activity_type,
        )
    except Exception as e:
        error_console.print(f"[red]Failed to invoke dispatch rerun API: {e}[/red]")
        raise typer.Exit(code=1)

    console.print("[green]DAG rerun successfully invoked.[/green]")


@app.command("dispatch-logs")
def get_dispatch_logs(
    dataset_id: Annotated[
        str,
        typer.Argument(help="The dataset ID to get dispatch logs."),
    ],
    export_id: Annotated[
        str,
        typer.Option(
            "--export-id",
            "-e",
            help="Export ID to filter logs (dispatch only)",
        ),
    ],
    profile: Annotated[
        str,
        typer.Option(help="profile to use: local, dev, staging, prod. Default: prod"),
    ] = None,
    delivery_id: Annotated[
        str,
        typer.Option("--delivery-id", "-d", help="Delivery ID to filter logs"),
    ] = None,
    page_size: Annotated[
        int,
        typer.Option("--page-size", help="Page size to limit logs"),
    ] = 100,
):
    """
    Retrieves dispatch logs for the provided dataset based on filter criteria.
    """
    if not profile:
        profile = get_current_profile()
    token: str = set_api_token(console, profile)
    if not token:
        raise typer.BadParameter("Failed to read Crux API token")

    mixpanel_token = get_mixpanel_token()
    track_mixpanel_event(
        "cruxctl dataset logs",
        {"Dataset ID": dataset_id},
        api_token=token,
        mixpanel_token=mixpanel_token,
        profile=profile,
    )

    filter_criteria = {
        "exportId": export_id,
        "deliveryId": delivery_id,
    }

    get_logs_with_pagination(
        profile, token, dataset_id, "dispatch", filter_criteria, page_size
    )


@app.command("pdk-logs")
def get_logs(
    dataset_id: Annotated[
        str,
        typer.Argument(help="The dataset ID to get PDK logs."),
    ],
    delivery_id: Annotated[
        str,
        typer.Option("--delivery-id", "-d", help="Delivery ID to filter PDK logs"),
    ],
    profile: Annotated[
        str,
        typer.Option(help="profile to use: local, dev, staging, prod. Default: prod"),
    ] = None,
    schedule_date: Annotated[
        str,
        typer.Option("--schedule-date", "-sd", help="Schedule date to filter PDK logs"),
    ] = None,
    page_size: Annotated[
        int,
        typer.Option("--page-size", help="Page size to limit logs"),
    ] = 100,
):
    """
    Retrieves PDK logs for the provided dataset based on filter criteria.
    """
    if not profile:
        profile = get_current_profile()
    token: str = set_api_token(console, profile)
    if not token:
        raise typer.BadParameter("Failed to read Crux API token")

    filter_criteria = {
        "scheduleDate": schedule_date,
        "deliveryId": delivery_id,
    }

    get_logs_with_pagination(
        profile, token, dataset_id, "pdk", filter_criteria, page_size
    )


def get_logs_with_pagination(
    profile: str,
    token: str,
    dataset_id: str,
    type: str,
    filter_criteria: dict,
    page_size: int,
):
    """
    Helper function to retrieve logs with pagination, and print log entries.
    """
    page_token = ""
    while True:
        try:
            # Make the API call with the filter criteria and pagination
            response = LogsHandler.get_logs(
                profile,
                token,
                dataset_id,
                type,
                **filter_criteria,
                page_token=page_token,
                page_size=page_size,
            )

            # Process the log entries
            if "entries" in response and response["entries"]:
                for entry in response["entries"]:
                    entry_time = entry.get("timestamp", "No timestamp available")
                    text_payload = entry.get("textPayload", None)
                    console.print(f"[{entry_time}] {text_payload}")
            else:
                console.print("No logs available for last 30 days.")

            # Pagination handling
            next_page_token = response.get("nextPageToken", None)
            if next_page_token:
                user_input = input("Press Enter to load the next page or 'q' to quit: ")
                if user_input.lower() == "q":
                    break
                page_token = next_page_token
            else:
                console.print("No more pages available.")
                break

        except Exception as e:
            error_console.print(f"[red]Failed to invoke logs API: {e}[/red]")
            raise typer.Exit(code=1)


def track_mixpanel_event(
    event_name: str, properties: dict, api_token: str, profile: str, mixpanel_token: str
) -> None:
    """
    Tracks an event in Mixpanel.
    :param event_name: The name of the event to track.
    :param properties: Properties to include with the event.
    :param api_token: The API token retrieved during the initial cruxctl command.
    :param profile: The profile being used (e.g., local, dev, staging, prod).
    """
    try:
        user_profile = get_user_info_by_token(api_token, profile)
        if "email" not in user_profile:
            return
        user_email = user_profile["email"]

        mp = Mixpanel(mixpanel_token)

        if platform.system() == "Darwin":
            properties["$os"] = "Mac OS X"
        else:
            properties["$os"] = platform.system()

        response = requests.get("https://ipinfo.io", timeout=5)
        if response.status_code == 200:
            location_data = response.json()
            properties["$city"] = location_data.get("city", "Unknown")
            properties["mp_country_code"] = location_data.get("country", "Unknown")

        mp.track(user_email, event_name, properties)
    except Exception:
        pass



from enum import Enum


class DatasetExpectedDeliveryDeadlineColumn(Enum):
    dataset_id = "dataset_id"
    deadline_minute = "deadline_minute"
    deadline_hour = "deadline_hour"
    deadline_day_of_the_month = "deadline_day_of_the_month"
    deadline_month = "deadline_month"
    deadline_day_of_week = "deadline_day_of_week"
    deadline_year = "deadline_year"
    timezone = "timezone"


class DatasetExpectedDeliveryDeadlineColumnCamelCase(Enum):
    dataset_id = "datasetId"
    deadline_minute = "deadlineMinute"
    deadline_hour = "deadlineHour"
    deadline_day_of_the_month = "deadlineDayOfTheMonth"
    deadline_month = "deadlineMonth"
    deadline_day_of_week = "deadlineDayOfWeek"
    deadline_year = "deadlineYear"
    timezone = "timezone"
    file_frequency = "fileFrequency"
    is_active = "isActive"
    is_excluded = "isExcluded"


class DatasetExpectedDeliveryDeadlineJoinedColumn(Enum):
    workflow_id = "workflow_id"
